<?php

namespace Teknisa\PLA\Modules\Parametros\Controllers;

use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;

class Pla03210UtilizacaoPratoSemana extends \Teknisa\GER\Controllers\TeknisaCrud{

    protected $crudService;
    protected $parameterBag;
    protected $util;
    protected $dataSourceManager;
    protected $connection;
    protected $RestrContrService;
    protected $errorMessages = array();
    protected $errors = array();

	public function __construct($crudService, $parameterBag, $util, $dataSourceManager, $connection, $RestrContrService) {
        $this->crudService = $crudService;
        $this->parameterBag = $parameterBag;
        $this->util = $util;
        $this->dataSourceManager = $dataSourceManager;
        $this->connection = $connection;
        $this->RestrContrService = $RestrContrService;
    }

    private function saveVigencia($vigencia, $response) {
        try {
            $dataSourceName = 'pla03210_vigutilpratoserv'; 
            $vigencia["NRVIGENCIA"] = $this->RestrContrService->getNewCode($dataSourceName);
            $params = array(
                'NRVIGENCIA'      => $vigencia['NRVIGENCIA'],
                'CDFILIAL'        => $vigencia['CDFILIAL'],
                'DTVIGENCIA'      => $vigencia['DTVIGENCIA'],
                'DTVIGENCIAFIM'   => $vigencia['DTVIGENCIAFIM'],
                'DESCRICAO'       => $vigencia['DESCRICAO'],
                'IDTIPORESTRICAO' => $vigencia['IDTIPORESTRICAO'],
                'NRORG'           => $this->crudService->getEnvironment()->getNrOrg()
            );
            $sql = "INSERT INTO VIGUTILPRATOSERV (
                NRVIGENCIA,
                CDFILIAL,
                DTVIGENCIA,
                DTVIGENCIAFIM,
                DESCRICAO,
                IDTIPORESTRICAO,
                NRORG
            ) VALUES (
                :NRVIGENCIA,
                :CDFILIAL,
                :DTVIGENCIA,
                :DTVIGENCIAFIM,
                :DESCRICAO,
                :IDTIPORESTRICAO,
                :NRORG
            )";
            $query = $this->connection->prepare($sql);
        	$query->execute($params);
            $response->addDataSet(new DataSet($dataSourceName, $params));
            return $vigencia['NRVIGENCIA'];
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveUtilPratoServ($utilpratoserv, $response) {
        try {
            $dataSourceName = "pla03210_util_prato_serv";
            $utilpratoserv["NRUTILPRATOSERV"] = $utilpratoserv['__is_new'] ? $this->RestrContrService->getNewCode($dataSourceName) : $utilpratoserv["NRUTILPRATOSERV"];
            $dataSet = new DataSet($dataSourceName, array($utilpratoserv));
            $request = new DTO\Request\DataSet($dataSet, "", "", "");
            $rowsToSave = $this->crudService->getRowsToSave($request, $dataSourceName);
            $nrutilpratoserv = $rowsToSave[0]["NRUTILPRATOSERV"];
            $dataSet = new DataSet($dataSourceName, $rowsToSave);
            $this->dataSourceManager->persist($dataSet);
            $response->addDataSet($dataSet);
            return $nrutilpratoserv;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function saveGrupoPrato(DTO\Request $request, DTO\Response $response) {
        try {
            $errors = array(
                'NRVIGENCIA' => '',
                'errors' => array()
            );
            $row = $request->getRow();
            $this->connection->beginTransaction();
            if (isset($row["vigencia"])) {
                $nrvigencia = $this->saveVigencia($row["vigencia"], $response);
            }
            foreach($row['NMGRUPOPRATO'] as $grupo){
                $row['NRGRUPOPRATOSERV'] = $grupo['NRGRUPOPRATOSERV'];
                try {
                    $grupo['NRVIGENCIA'] = isset($row["vigencia"]) ? $nrvigencia : $row['NRVIGENCIA'];
                    $row['NRVIGENCIA'] = isset($row["vigencia"])  ? $nrvigencia : $row['NRVIGENCIA'];
                    $grupo['CDSERVFILI'] = $row['CDSERVFILI'];
                    $this->util->validateDuplicatedRecords($grupo, "pla03210_util_prato_serv", array("CDSERVFILI", "NRGRUPOPRATOSERV", "NRVIGENCIA"));
                    $this->saveUtilPratoServ($row, $response);
                } catch(\Exception $e) {
                    if ($e->getMessage() == "registerExists") {
                        $vigencia = isset($row['NRVIGENCIA']) ? $row['NRVIGENCIA'] : $nrvigencia;
                        $errors['NRVIGENCIA'] = $vigencia;
                        array_push($errors['errors'], array("NMGRUPOPRATO" => $grupo["NMGRUPOPRATO"]));
                    } else {
                        throw $e;
                    }
                }
            }

            $this->connection->commit();
            if (!empty($errors['errors'])) {
                $response->addDataSet(new DataSet("pla03210_vigutilpratoserv", $errors));
            }
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function saveEditGrupoPrato(DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $this->connection->beginTransaction();
        try{
            $this->saveUtilPratoServ($row, $response);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw $e;
        }
    }

    public function deleteServicos(DTO\Request $request, DTO\Response $response) {
        try {
            $this->connection->beginTransaction();
            
            $row = $request->getRow();
            foreach ($row['servicos'] as $servico) {
                $this->deleteServ($row["NRVIGENCIA"], $servico["CDSERVFILI"]);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function deleteGrupoPrato(DTO\Request $request, DTO\Response $response){
        try {
            $this->connection->beginTransaction();
            
            $row = $request->getRow();
            $grupos = $row['grupos'];
            foreach ($grupos as $grupo) {
                $this->deleteGrupo($grupo['NRUTILPRATOSERV'], $row["NRVIGENCIA"], $row["CDSERVFILI"]);
            }
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function deleteGrupo($nrutilpratoserv, $nrvigencia, $cdservfili) {
        $sql = "DELETE FROM UTILPRATOSERV WHERE NRUTILPRATOSERV = :NRUTILPRATOSERV AND NRVIGENCIA = :NRVIGENCIA AND CDSERVFILI = :CDSERVFILI AND NRORG = :NRORG";
        $params = array(
            "NRUTILPRATOSERV" => $nrutilpratoserv,
            "NRVIGENCIA"  => $nrvigencia,
            "CDSERVFILI"  => $cdservfili,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    private function deleteServ($nrvigencia, $cdservfili) {
        $sql = "DELETE FROM UTILPRATOSERV WHERE NRVIGENCIA = :NRVIGENCIA AND CDSERVFILI = :CDSERVFILI AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "CDSERVFILI"  => $cdservfili,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
     public function deleteAllServVig(DTO\Request $request, DTO\Response $response){
        try {
            $this->connection->beginTransaction();
            
            $row = $request->getRow();
            $this->deleteAllServ($row["NRVIGENCIA"]);
            $this->deleteVig($row['CDFILIAL'], $row['NRVIGENCIA']);
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    private function deleteAllServ($nrvigencia) {
        $sql = "DELETE FROM UTILPRATOSERV WHERE NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }
    
    private function deleteVig($cdfilial, $nrvigencia) {
        $sql = "DELETE FROM VIGUTILPRATOSERV WHERE CDFILIAL = :CDFILIAL AND NRVIGENCIA = :NRVIGENCIA AND NRORG = :NRORG";
        $params = array(
            "CDFILIAL"  => $cdfilial,
            "NRVIGENCIA"  => $nrvigencia,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    }

    public function updateVigencia (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $sqlUpdate = "UPDATE VIGUTILPRATOSERV SET DTVIGENCIAFIM =:DTVIGENCIAFIM, DESCRICAO = :DESCRICAO WHERE CDFILIAL =:CDFILIAL AND NRVIGENCIA =:NRVIGENCIA";
        
        $std = $this->connection->prepare($sqlUpdate);
        $params = array(
            'DTVIGENCIAFIM' => $row['DTVIGENCIAFIM'], 
            'DESCRICAO'     => $row['DESCRICAO'],
            'CDFILIAL'      => $row['CDFILIAL'],
            'NRVIGENCIA'    => $row['NRVIGENCIA']
        );
        $std->execute($params);
        
    }
    
    public function verifyExistsVig (DTO\Request $request, DTO\Response $response){
        $row = $request->getRow();
        $sql = "SELECT
                    DTVIGENCIA
                FROM
                    VIGUTILPRATOSERV
                WHERE
                    CDFILIAL            = :CDFILIAL
                    AND DTVIGENCIA      = :DTVIGENCIA
                    AND IDTIPORESTRICAO = :IDTIPORESTRICAO";
        $std = $this->connection->prepare($sql);
        $params = array(
            'CDFILIAL'        => $row['CDFILIAL'], 
            'DTVIGENCIA'      => $row['DTVIGENCIA'],
            'IDTIPORESTRICAO' => $row['IDTIPORESTRICAO']
        );
        $std->execute($params);
        $result = $std->fetchAll();
        $response->addDataSet(new DataSet("vigencia", $result));
    }
    
    //Importação - Exportação
    public function servicosImportExport(DTO\Request $request, DTO\Response $response) {
        try {
            $row = $request->getRow();
            $this->connection->beginTransaction();

            $servicos = $row["NMSERVICO"];
            if (isset($row["vigencia"])) {
                $nrvigenciadest = $this->saveVigencia($row["vigencia"], $response);
            } else {
                $nrvigenciadest =  $row["NRVIGENCIA"];
            }

            $nrvigenciaorig =  $row["NRVIGENCIAORIG"];
            $tpoper = $row["TPOPER"];
            foreach ($servicos as $servico) {
                if ($tpoper == "S") {
                    $this->deleteUtilPratoServ($nrvigenciadest, $servico["CDSERVFILI"]);
                }
                $grupos = $this->getUtilPratoServ($nrvigenciaorig, $servico["CDSERVFILI"]);
                $gruposExisting = $this->getUtilPratoServ($nrvigenciadest, $servico["CDSERVFILI"]);
                $gruposExisting = array_map(function($grupo){
                    return $grupo["CDGRUPOPRATO"].'|'.$grupo['NRGRUPOPRATO'].'|'.$grupo["NRGRUPOPRATOSERV"];
                }, $gruposExisting);
                $gruposToSave = array();
                foreach ($grupos as &$grupo) {
                    if (in_array($grupo["CDGRUPOPRATO"].'|'.$grupo['NRGRUPOPRATO'].'|'.$grupo["NRGRUPOPRATOSERV"], $gruposExisting)) {
                        continue;
                    } else {
                        $grupo["NRUTILPRATOSERV"] = $this->RestrContrService->getNewCode("UTILPRATOSERV");
                        $grupo["NRVIGENCIA"] = $nrvigenciadest;
                        $grupo["__is_new"] = true;
                        array_push($gruposToSave, $grupo);
                    }
                }
                $dataSourceName = "pla03210_util_prato_serv";
                $dataSet = new DataSet($dataSourceName, $gruposToSave);
                $newRequest = new DTO\Request\DataSet($dataSet, "", "", "");
                $rowsToSave = $this->crudService->getRowsToSave($newRequest, $dataSourceName);
                $dataSet = new DataSet($dataSourceName, $rowsToSave);
                $this->dataSourceManager->persist($dataSet);
            }

            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            $response->setError(new Error($e->getMessage(), 500));
        }
    }
    
    public function getUtilPratoServ($nrvigencia, $cdservfili) {
        $sql = "SELECT
                    UT.NRUTILPRATOSERV,
                    UT.NRVIGENCIA,
                    UT.CDSERVFILI,
                    UT.NRGRUPOPRATOSERV,
                    UT.IDUTILPRATOSERVSEG,
                    UT.IDUTILPRATOSERVTER,
                    UT.IDUTILPRATOSERVQUA,
                    UT.IDUTILPRATOSERVQUI,
                    UT.IDUTILPRATOSERVSEX,
                    UT.IDUTILPRATOSERVSAB,
                    UT.IDUTILPRATOSERVDOM,
                    UT.IDUTILPRATOSERVFERI,
                    UT.IDUTILPRATOSERVPRIM,
                    UT.IDUTILPRATOSERVULT,
                    UT.NRORG,
                    G.CDFILIAL,
                    G.CDGRUPOPRATO,
                    G.NMGRUPOPRATO,
                    GS.NRGRUPOPRATO
                FROM
                    UTILPRATOSERV UT
                    INNER JOIN GRUPOPRATOSERV GS ON UT.NRGRUPOPRATOSERV = GS.NRGRUPOPRATOSERV
                    INNER JOIN GRUPOPRATO G ON GS.NRGRUPOPRATO = G.NRGRUPOPRATO
                WHERE
                    UT.NRVIGENCIA =:NRVIGENCIA
                    AND UT.CDSERVFILI =:CDSERVFILI
                    AND UT.NRORG = :NRORG";
        $params = array(
            'NRVIGENCIA' => $nrvigencia,
            'CDSERVFILI' => $cdservfili,
            'NRORG'      => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
        return $query->fetchAll();
    }
    
    private function deleteUtilPratoServ($nrvigencia, $cdservfili) {
        $this->connection->beginTransaction();
        $sql = "DELETE FROM UTILPRATOSERV WHERE NRVIGENCIA = :NRVIGENCIA AND CDSERVFILI = :CDSERVFILI AND NRORG = :NRORG";
        $params = array(
            "NRVIGENCIA"  => $nrvigencia,
            "CDSERVFILI"  => $cdservfili,
            "NRORG"       => $this->crudService->getEnvironment()->getNrOrg()
        );
        $query = $this->connection->prepare($sql);
    	$query->execute($params);
    	$this->connection->commit();
    }

}