<?php
namespace Teknisa\PLA\Modules\Relatorio\Controllers;

// use Teknisa\GER\Services\NewCode;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
// use Teknisa\GER\Controllers\Consplan;
// use Teknisa\GER\Utilities\Util;

class Pla09600AcompanhamentoDePU extends \Zeedhi\Framework\Controller\Report {

    protected $novoCodigo;
    protected $procedureConsumoPlanejado;
    protected $util;
    protected $consplan;
    protected $nrorg;
    protected $cdfilial;
    protected $nrconsplan;
    protected $environment;
    protected $connection;
    protected $birtStrategy;

    public function __construct($reportService, $novoCodigo,  $procedureConsumoPlanejado, $util,  $consplan, $environment, $connection, $birtStrategy) {

        $this->novoCodigo = $novoCodigo;
        $this->procedureConsumoPlanejado = $procedureConsumoPlanejado;
        $this->util = $util;
        $this->consplan = $consplan;
        $this->environment = $environment;
        $this->connection = $connection;
        $this->birtStrategy = $birtStrategy;
        parent::__construct($reportService);
    }

    protected $reportMapping = array (
        'PLA09600' =>
        array (
            'strategy' => 'BIRT',
            'parameterMapping' =>
            array (
                'CDFILIAL'        => 'P_CDFILIAL',
                'NRCONSPLAN'      => 'P_NRCONSPLAN',
                'NRTIPOCUSTOPLAN' => 'P_NRTIPOCUSTOPLAN',
                'NRTIPOCUSTOREAL' => 'P_NRTIPOCUSTOREAL',
                'LISTASERV'       => 'P_CDSERVFILI',
                'TCDSERVFILI'     => 'P_TCDSERVFILI',
                'DTINICIAL'       => 'P_DTINICIAL',
                'DTFINAL'         => 'P_DTFINAL',
                'DTPROC'          => 'P_DTPROC',
                'NRORG' => 'P_NRORG'
            ),
        ),
    );

     public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();

        $date        = \DateTime::createFromFormat('m/Y', $row['MES']);
        $dateInicial = \DateTime::createFromFormat('d/m/Y', $row['DTINICIAL']);
        $dateFinal   = \DateTime::createFromFormat('d/m/Y', $row['DTFINAL']);
        $row['CDOPERADOR'] = $this->environment->getCdOperador();
        $row['NRORG'] = $this->environment->getNrOrgTrab();

        $codigo = $this->novoCodigo->getCodeByTableName('CONSPLAN'.$row['CDFILIAL'], $row['NRORG']);

        $this->procedureConsumoPlanejado->addParam(new Param('P_FILIAL',        Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_SERVICOS',      Param::PARAM_INPUT, $row['LISTASERV'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTINICIAL',     Param::PARAM_INPUT, $row['DTINICIAL'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_DTFINAL',       Param::PARAM_INPUT, $row['DTFINAL'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARQUIVO',     Param::PARAM_INPUT, "T", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDPRODUTOS',    Param::PARAM_INPUT, "CFP", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDSEPPRATO',    Param::PARAM_INPUT, "P", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_PEACRESCIMO',   Param::PARAM_INPUT, 0, Param::PARAM_TYPE_INT));
        $this->procedureConsumoPlanejado->addParam(new Param('P_IDARREDOND',    Param::PARAM_INPUT, "N", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_NRCONSPLAN',    Param::PARAM_INPUT, $codigo, Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOINI',  Param::PARAM_INPUT, ' ', Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDPRODUTOFIN',  Param::PARAM_INPUT, "zzzzzzzzzz", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_CDOPERADOR',    Param::PARAM_INPUT, $row['CDOPERADOR'], Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTINICIAL',   Param::PARAM_INPUT, "_".$dateInicial->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $this->procedureConsumoPlanejado->addParam(new Param('P_STDTFINAL',     Param::PARAM_INPUT, "_".$dateFinal->format('Ymd')."_", Param::PARAM_TYPE_STR));
        $this->params = array();

        $resultado = $this->procedureConsumoPlanejado->execute();
        $nrConsPlan = $codigo;

        $row['NRCONSPLAN'] = $nrConsPlan;
        $row['NRTIPOCUSTOPLAN'] =  (int)$row['NRTIPOCUSTOPLAN'];
        $row['NRTIPOCUSTOREAL'] =  (int)$row['NRTIPOCUSTOREAL'];

        $dataProc = $this->util->retornaDataProcessamento($row['CDFILIAL']);
        $dataProc = explode(' ', $dataProc);
        $row['DTPROC'] = $dataProc[0];
        if($row['TCDSERVFILI']== "T"){
            $row['LISTASERV']= null;
        }else{
            $row['TCDSERVFILI'] = "X";
        }
        
        $this->nrorg = $row['NRORG'];
        $this->cdfilial = $row['CDFILIAL'];
        $this->nrConsPlan = $nrConsPlan;
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
    }

    // public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
     
    //     $stmt = $this->connection->prepare("DELETE FROM CONSPLAN WHERE NRORG = :NRORG AND CDFILICPLA = :CDFILICPLA AND NRCONSPLAN = :NRCONSPLAN");
    //     $stmt->execute(array(
    //         "NRORG" => $this->nrorg,
    //         "CDFILICPLA" => $this->cdfilial,
    //         "NRCONSPLAN" => $this->nrConsPlan
    //     ));
    // }
}