<?php
namespace Teknisa\PLA\Modules\Requisicao\Services;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;

class Pla01800RequisicaoGeralRepository {

	protected $connection;
    protected $requisicaoAnaliticaSetorPla;
    protected $entityManager;

    public function __construct($crudService, $connection, StoredProcedure $requisicaoAnaliticaSetorPla, $entityManager){
        $this->crudService = $crudService;
    	$this->connection = $connection;
        $this->requisicaoAnaliticaSetorPla = $requisicaoAnaliticaSetorPla;
    	$this->entityManager = $entityManager;
    }

    public function verificaParametrizacao($nrlistaFiliServ){
        $param = array("NRLISTA" => $nrlistaFiliServ);
        $query = "SELECT DISTINCT P.CDTPRCONSFIX, P.CDPRACONSFIX, P.CDTPRCONSPER, P.CDPRACONSPER
                  FROM SERVICO S
                  INNER JOIN PARAMPLANEJA P
                    ON P.CDFILIAL = S.CDFILIAL
                  INNER JOIN LISTA L
                    ON S.CDFILIAL || S.CDSERVFILI = L.CDLISTA
                  WHERE L.NRLISTA = :NRLISTA
                    AND S.IDUTRETIPRAT = 'S'";
        $resultados = $this->connection->fetchAll($query, $param);
        foreach ($resultados as $resultado) {
            if(empty($resultado["CDTPRCONSFIX"])){
                return "TP_CONSFIXO";
            }
            if(empty($resultado["CDPRACONSFIX"])){
                return "PRATO_CONSFIXO";
            }
            if(empty($resultado["CDTPRCONSPER"])){
                return "TP_CONSPER";
            }
            if(empty($resultado["CDPRACONSPER"])){
                return "PRATO_CONSPER";
            }
        }
        return "";
    }

    public function deleteConsplan($nrconsplan){
        $param = array("NRCONSPLAN" => $nrconsplan);
        $query = "DELETE FROM CONSPLAN
                    WHERE NRCONSPLAN = :NRCONSPLAN";
        $this->connection->executeQuery($query, $param);
    }

    public function deleteConsplanLista($nrconsplan, $nrlistaFiliServ){
        $param = array("NRCONSPLAN" => $nrconsplan, "NRLISTA" => $nrlistaFiliServ);
        $query = "DELETE FROM CONSPLAN
                    WHERE NRCONSPLAN = :NRCONSPLAN
                    AND EXISTS (
                        SELECT DISTINCT SUBSTR(CDLISTA, 1, 4) AS CDFILICPLA
                        FROM LISTA
                        WHERE NRLISTA = :NRLISTA
                    )";
        $this->connection->executeQuery($query, $param);
    }

    public function executaRequisicao($param){
        date_default_timezone_set("America/Sao_Paulo");

        $listafilial = $param["LISTAFILIAL"];
        $listaservicos = $param["LISTASERVFILI"];
        $dtinicial = $param["DTINICIAL"];
        $dtfinal = $param["DTFINAL"];
        $peacrescimo = $param["PEACRESCIMO"];
        $cdoperador = $param["CDOPERADOR"];
        $dhrcalcpr = date('d/m/Y');
        $nrconsplan = $param["NRCONSPLAN"];
        $idarquivo = 'A';
        $idprodutos = $param["IDCONSUMO"];
        $idsepprato = 'P';
        $idarredond = $param["IDARREDOND"];
        $cdprodutoini = $param["CDPRODINI"];
        $cdprodutofin = $param["CDPRODFIN"];
        $stdtinicial = null;
        $stdfinal = null;
        $nrconsplan[0] = 'R';
    
        if((strlen($listafilial) == 4))
            $listafilial.="_";
        
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_FILIAL', Param::PARAM_INPUT, $listafilial, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_SERVICOS', Param::PARAM_INPUT, $listaservicos, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_DTINICIAL', Param::PARAM_INPUT, $dtinicial, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_DTFINAL', Param::PARAM_INPUT, $dtfinal, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_IDARQUIVO', Param::PARAM_INPUT, $idarquivo, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_IDPRODUTOS', Param::PARAM_INPUT, $idprodutos, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_IDSEPPRATO', Param::PARAM_INPUT, $idsepprato, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_PEACRESCIMO', Param::PARAM_INPUT, $peacrescimo, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_IDARREDOND', Param::PARAM_INPUT, $idarredond, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_NRCONSPLAN', Param::PARAM_INPUT, $nrconsplan, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_CDPRODUTOINI', Param::PARAM_INPUT, $cdprodutoini, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_CDPRODUTOFIN', Param::PARAM_INPUT, $cdprodutofin, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_CDOPERADOR', Param::PARAM_INPUT, $cdoperador, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_DTHRCALCPR', Param::PARAM_INPUT, $dhrcalcpr, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_STDTINICIAL', Param::PARAM_INPUT, $stdtinicial, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->addParam(
            new Param('P_STDTFINAL', Param::PARAM_INPUT, $stdfinal, Param::PARAM_TYPE_STR)
        );
        $this->requisicaoAnaliticaSetorPla->execute();
    }

    public function saveLista($param){
        $dataSet = new DataSet('lista', [$param]);
        $request = new DTO\Request\DataSet($dataSet, null, null, null);
        $row =$this->crudService->getRowsToSave($request, 'lista');
        $this->crudService->save('lista',$row);
    }

    public function deleteLista($nrLista, $nrorg=1){
        $query="DELETE FROM LISTA WHERE NRLISTA = :NRLISTA AND NRORG=:NRORG";
        $this->connection->executeQuery($query, array("NRLISTA"=>$nrLista, "NRORG"=>$nrorg));
    }
    
    public function verificaLista($nrlistaFiliServ){
    $sql = "SELECT IDUTRETIPRAT
            FROM SERVICO, LISTA
            WHERE LISTA.NRLISTA = :P_NRLISTA
            AND SERVICO.CDFILIAL = SUBSTR(LISTA.CDLISTA, 1, 4)
            AND SERVICO.CDSERVFILI = SUBSTR(LISTA.CDLISTA, 5, 5)
            AND SERVICO.IDUTRETIPRAT = 'N'";
    $params = array("P_NRLISTA" => $nrlistaFiliServ);
	$query = $this->entityManager->getConnection()->prepare($sql);
    $query->execute($params);
    $result = $query->fetchAll();
    return $result;
  }
  
  public function getProductMask(){
      $sql = 'SELECT CDPICTPROD FROM PARAMGERAL';
      $stmt = $this->entityManager->getConnection()->prepare($sql);
      $stmt->execute();
      return $stmt->fetch()['CDPICTPROD'];
  }
}